@@ -1,21 +1,24 @@
 # --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
 #
-# This file is part of the tsfpga project.
+# This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
 # https://gitlab.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
+# Standard libraries
 import fnmatch
 import time
 from pathlib import Path
 from threading import Lock
 
-from vunit.test.list import TestList
-from vunit.test.runner import TestRunner
-from vunit.test.report import TestReport, TestResult
+# Third party libraries
 from vunit.color_printer import COLOR_PRINTER, NO_COLOR_PRINTER
+from vunit.test.list import TestList
+from vunit.test.report import TestReport, TestResult
+from vunit.test.runner import TestRunner
 
+# First party libraries
 from tsfpga.system_utils import create_directory, read_last_lines_of_file
 
 
@@ -267,6 +270,12 @@ class BuildProjectList:
                         if fnmatch.filter([project.name], project_filter):
                             yield project
 
+                            # Do not continue with further filters if we have already matched this
+                            # project.
+                            # Multiple filters might match the same project, and multiple objects
+                            # of the same project will break build
+                            break
+
 
 class BuildProjectCreateWrapper:
 
@@ -478,7 +487,6 @@ class BuildReport(TestReport):
 
 
 class BuildResult(TestResult):
-
     report_length_lines = None
 
     def _print_output(self, printer, num_lines):
