@@ -1,18 +1,22 @@
 # --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
 #
-# This file is part of the tsfpga project.
+# This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
 # https://gitlab.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
+# Standard libraries
 from pathlib import Path
 
-from tsfpga.constraint import Constraint
-from tsfpga.module import BaseModule, get_hdl_modules
-from tsfpga.examples.example_env import get_tsfpga_example_modules
-from tsfpga.vivado.project import VivadoProject
+# Import before others since it modifies PYTHONPATH. pylint: disable=unused-import
+import tsfpga.examples.example_pythonpath  # noqa: F401
 
+# First party libraries
+from tsfpga.constraint import Constraint
+from tsfpga.examples.example_env import get_hdl_modules, get_tsfpga_example_modules
+from tsfpga.module import BaseModule
+from tsfpga.vivado.project import VivadoProject
 
 THIS_FILE = Path(__file__)
 
