@@ -1,11 +1,12 @@
 # --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
 #
-# This file is part of the tsfpga project.
+# This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
 # https://gitlab.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
+# First party libraries
 from tsfpga.module import BaseModule
 
 
@@ -18,5 +19,6 @@ class Module(BaseModule):
             self.path / "src" / "multiplication.vhd",
             self.path / "test" / "tb_multiplication.vhd",
         }
-        files_avoid = {} if include_ip_cores else files_that_depend_on_ip_cores
+        files_avoid = None if include_ip_cores else files_that_depend_on_ip_cores
+
         return super().get_simulation_files(files_avoid=files_avoid, **kwargs)
