@@ -1,23 +1,21 @@
 # --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
 #
-# This file is part of the tsfpga project.
+# This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
 # https://gitlab.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
+# Standard libraries
 import argparse
-from shutil import which
-import sys
 from pathlib import Path
+from shutil import which
 
-# Do PYTHONPATH insert() instead of append() to prefer any local repo checkout over any pip install
-PATH_TO_TSFPGA = Path(__file__).parent.parent.resolve()
-sys.path.insert(0, str(PATH_TO_TSFPGA))
-
-from vunit import VUnitCLI, VUnit
-from vunit.vivado.vivado import create_compile_order_file, add_from_compile_order_file
+# Third party libraries
+from vunit import VUnit, VUnitCLI
+from vunit.vivado.vivado import add_from_compile_order_file, create_compile_order_file
 
+# First party libraries
 import tsfpga
 import tsfpga.create_vhdl_ls_config
 from tsfpga.module import ModuleList
@@ -82,20 +80,22 @@ class SimulationProject:
     Class for setting up and handling a VUnit simulation project. Should be reusable in most cases.
     """
 
-    def __init__(self, args, preprocessing_disable=False):
+    def __init__(self, args, enable_preprocessing=False):
         """
         Create a VUnit project, configured according to the given arguments.
 
         Arguments:
-            preprocessing_disable (bool): If ``True``, VUnit location/check preprocessing will not
+            enable_preprocessing (bool): If ``True``, VUnit location/check preprocessing will
                 be enabled.
-
-        Return: The created VUnit project.
         """
-        self.vunit_proj = VUnit.from_args(args=args)
+        # Note: compile_builtins=False can be removed when VUnit 5.0 is released:
+        # https://github.com/VUnit/vunit/issues/777
+        self.vunit_proj = VUnit.from_args(args=args, compile_builtins=False)
+        self.vunit_proj.add_vhdl_builtins()
         self.vunit_proj.add_verification_components()
         self.vunit_proj.add_random()
-        if not preprocessing_disable:
+
+        if enable_preprocessing:
             self.vunit_proj.enable_location_preprocessing()
             self.vunit_proj.enable_check_preprocessing()
 
@@ -117,6 +117,7 @@ class SimulationProject:
         """
         modules_no_sim = ModuleList() if modules_no_sim is None else modules_no_sim
 
+        include_unisim = not args.vivado_skip
         include_ip_cores = self.has_commercial_simulator and not args.vivado_skip
 
         for module in modules + modules_no_sim:
@@ -126,7 +127,9 @@ class SimulationProject:
             simulate_this_module = module not in modules_no_sim
 
             for hdl_file in module.get_simulation_files(
-                include_tests=simulate_this_module, include_ip_cores=include_ip_cores
+                include_tests=simulate_this_module,
+                include_unisim=include_unisim,
+                include_ip_cores=include_ip_cores,
             ):
                 if hdl_file.is_vhdl or hdl_file.is_verilog_source:
                     vunit_library.add_source_file(hdl_file.path)
@@ -134,9 +137,63 @@ class SimulationProject:
                     assert False, f"Can not handle this file: {hdl_file}"
 
             if simulate_this_module:
-                module.setup_vunit(vunit_proj=self.vunit_proj, **setup_vunit_kwargs)
+                module.setup_vunit(
+                    vunit_proj=self.vunit_proj,
+                    include_unisim=include_unisim,
+                    include_ip_cores=include_ip_cores,
+                    **setup_vunit_kwargs,
+                )
 
-    def add_vivado_simlib_and_ip_cores(
+    def add_vivado_simlib(self, args):
+        """
+        Add Vivado simlib to the VUnit project, unless instructed not to by ``args``.
+        Will compile simlib if necessary.
+
+        Arguments:
+            args: Command line argument namespace from ``simulate.py``.
+
+        Return:
+            :class:`.VivadoSimlibCommon`: The simlib object.
+        """
+        if args.vivado_skip:
+            return None
+
+        return self._add_simlib(
+            output_path=args.output_path_vivado, force_compile=args.simlib_compile
+        )
+
+    def _add_simlib(self, output_path, force_compile):
+        """
+        Add Vivado simlib to the VUnit project. Compile if needed.
+
+        .. note::
+
+            This method can be overloaded in a child class if you want to do something more
+            advanced, e.g. fetching compiled simlib from Artifactory.
+
+        Arguments:
+            output_path (pathlib.Path): Compiled simlib will be placed in sub-directory of
+                this path.
+            force_compile (bool): Will (re)-compile simlib even if compiled artifacts exist.
+
+        Return:
+            VivadoSimlibCommon: The simlib object.
+        """
+        vivado_simlib = VivadoSimlib.init(output_path, self.vunit_proj)
+        if force_compile or vivado_simlib.compile_is_needed:
+            vivado_simlib.compile()
+            vivado_simlib.to_archive()
+
+        vivado_simlib.add_to_vunit_project()
+
+        # Code in the "vital2000" package gives GHDL errors such as "result subtype of a pure
+        # function cannot have access subelements". Hence, relaxed rules need to be enabled when
+        # using unisim.
+        self.vunit_proj.set_sim_option("ghdl.elab_flags", ["-frelaxed-rules"])
+
+        return vivado_simlib
+
+    def add_vivado_ip_cores(
         self,
         args,
         modules,
@@ -144,13 +201,11 @@ class SimulationProject:
         vivado_ip_core_project_class=None,
     ):
         """
-        Add Vivado simlib and module IP cores to the VUnit project.
-        Will compile and add simlib unless instructed not to by ``args``.
-        When running with a commercial simulator, IP cores will be generated and added,
-        unless instructed not to by ``args``.
+        Generate IP cores from the modules, unless instructed not to by ``args``.
+        When running with a commercial simulator they will be added to the VUnit project.
 
         Arguments:
-            args: Command line argument namespace.
+            args: Command line argument namespace from ``simulate.py``.
             modules (:class:`.ModuleList`): IP cores from these modules will be included in the
                 simulation project.
             vivado_part_name (str): Part name to be used for Vivado IP core project. Might have to
@@ -165,8 +220,6 @@ class SimulationProject:
         if args.vivado_skip:
             return None
 
-        self._add_simlib(output_path=args.output_path_vivado, force_compile=args.simlib_compile)
-
         # Generate IP core simulation files. Might be used for the vhdl_ls config,
         # even if they are not added to the simulation project.
         (
@@ -186,27 +239,6 @@ class SimulationProject:
 
         return ip_core_vivado_project_directory
 
-    def _add_simlib(self, output_path, force_compile):
-        """
-        Add Vivado simlib to the VUnit project. Compile if needed.
-
-        .. note::
-
-            This method can be overloaded in a child class if you want to do something more
-            advanced, e.g. fetching compiled simlib from Artifactory.
-
-        Arguments:
-            vunit_proj: Vivado simlib will be added to this VUnit project.
-            output_path (pathlib.Path): Compiled simlib will be placed in sub-directory of
-                this path.
-            force_compile (bool): Will (re)-compile simlib even if compiled artifacts exist.
-        """
-        vivado_simlib = VivadoSimlib.init(output_path, self.vunit_proj)
-        if force_compile or vivado_simlib.compile_is_needed:
-            vivado_simlib.compile()
-            vivado_simlib.to_archive()
-        vivado_simlib.add_to_vunit_project()
-
     @staticmethod
     def _generate_ip_core_files(
         modules, output_path, force_generate, part_name, vivado_project_class=None
@@ -246,7 +278,7 @@ class SimulationProject:
 
 
 def create_vhdl_ls_configuration(
-    output_path, temp_files_path, modules, ip_core_vivado_project_directory
+    output_path, temp_files_path, modules, ip_core_vivado_project_directory=None
 ):
     """
     Create config for vhdl_ls (https://github.com/VHDL-LS/rust_hdl).
@@ -265,7 +297,12 @@ def create_vhdl_ls_configuration(
     # If we were to use the "real" VUnit project that we set up above instead, all the files would
     # be in the "preprocessed" folder. Hence we use an "empty" VUnit project, and add all files
     # via modules.
-    vunit_proj = VUnit.from_argv(argv=["--output-path", str(temp_files_path / "vhdl_ls_vunit_out")])
+    # Note: compile_builtins=False can be removed when VUnit 5.0 is released:
+    # https://github.com/VUnit/vunit/issues/777
+    vunit_proj = VUnit.from_argv(
+        argv=["--output-path", str(temp_files_path / "vhdl_ls_vunit_out")], compile_builtins=False
+    )
+    vunit_proj.add_vhdl_builtins()
     vunit_proj.add_verification_components()
     vunit_proj.add_random()
     vunit_proj.add_osvvm()
