@@ -1,14 +1,18 @@
 # --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
 #
-# This file is part of the tsfpga project.
+# This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
 # https://gitlab.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
+# Standard libraries
+import random
+
+# Third party libraries
 from hdl_registers.parser import from_toml
 
-import tsfpga
+# First party libraries
 from tsfpga.constraint import Constraint
 from tsfpga.hdl_file import HdlFile
 from tsfpga.ip_core_file import IpCoreFile
@@ -20,7 +24,7 @@ class BaseModule:
     """
     Base class for handling a HDL module with RTL code, constraints, etc.
 
-    Files are gathered from a lot of different subfolders, to accommodate for projects having
+    Files are gathered from a lot of different sub-folders, to accommodate for projects having
     different catalog structure.
     """
 
@@ -29,7 +33,7 @@ class BaseModule:
         Arguments:
             path (pathlib.Path): Path to the module folder.
             library_name (str): VHDL library name.
-            default_registers (list(Register)): Default registers.
+            default_registers (list(hdl_registers.register.Register)): Default registers.
         """
         self.path = path.resolve()
         self.name = path.name
@@ -68,7 +72,7 @@ class BaseModule:
 
         return files
 
-    def _get_hdl_file_list(self, folders, files_include, files_avoid):
+    def _get_hdl_file_list(self, folders, files_include=None, files_avoid=None):
         """
         Return a list of HDL file objects.
         """
@@ -85,8 +89,8 @@ class BaseModule:
     @property
     def registers(self):
         """
-        :class:`.RegisterList`: Get the registers for this module. Can be ``None`` if no TOML file
-            exists and no hook creates registers.
+        hdl_registers.register_list.RegisterList: Get the registers for this module. Can be
+            ``None`` if no TOML file exists and no hook creates registers.
         """
         if self._registers is not None:
             # Only create object once
@@ -120,8 +124,50 @@ class BaseModule:
         if self.registers is not None:
             self.registers.create_vhdl_package(self.path)
 
-    # pylint: disable=unused-argument
-    def get_synthesis_files(self, files_include=None, files_avoid=None, **kwargs):
+    @property
+    def synthesis_folders(self):
+        """
+        Synthesis/implementation source code files will be gathered from these folders.
+
+        Return:
+            list(pathlib.Path): Folder paths.
+        """
+        return [
+            self.path,
+            self.path / "src",
+            self.path / "rtl",
+            self.path / "hdl" / "rtl",
+            self.path / "hdl" / "package",
+        ]
+
+    @property
+    def sim_folders(self):
+        """
+        Files with simulation models (the ``sim`` folder) will be gathered from these folders.
+
+        Return:
+            list(pathlib.Path): Folder paths.
+        """
+        return [
+            self.path / "sim",
+        ]
+
+    @property
+    def test_folders(self):
+        """
+        Testbench files will be gathered from these folders.
+
+        Return:
+            list(pathlib.Path): Folder paths.
+        """
+        return [
+            self.path / "test",
+            self.path / "rtl" / "tb",
+        ]
+
+    def get_synthesis_files(
+        self, files_include=None, files_avoid=None, **kwargs
+    ):  # pylint: disable=unused-argument
         """
         Get a list of files that shall be included in a synthesis project.
 
@@ -143,15 +189,8 @@ class BaseModule:
         """
         self.create_regs_vhdl_package()
 
-        folders = [
-            self.path,
-            self.path / "src",
-            self.path / "rtl",
-            self.path / "hdl" / "rtl",
-            self.path / "hdl" / "package",
-        ]
         return self._get_hdl_file_list(
-            folders=folders, files_include=files_include, files_avoid=files_avoid
+            folders=self.synthesis_folders, files_include=files_include, files_avoid=files_avoid
         )
 
     def get_simulation_files(
@@ -162,8 +201,8 @@ class BaseModule:
         and ``files_avoid``.
 
         Arguments:
-            include_tests (bool): When ``False`` the ``test`` folder, where testbenches usually
-                reside, is not included. The ``sim`` folder is always included.
+            include_tests (bool): When ``False``, the ``test`` files are not included
+                (the ``sim`` files are always included).
             files_include (set(`pathlib.Path`)): Optionally filter to only include these files.
             files_avoid (set(`pathlib.Path`)): Optionally filter to discard these files.
             kwargs: Further parameters that can be sent by simulation flow to control what
@@ -172,43 +211,38 @@ class BaseModule:
         Return:
             list(HdlFile): Files that should be included in a simulation project.
         """
-        test_folders = [
-            self.path / "sim",
-        ]
+        test_folders = self.sim_folders.copy()
 
         if include_tests:
-            test_folders += [self.path / "rtl" / "tb", self.path / "test"]
+            test_folders += self.test_folders
+
+        test_files = self._get_hdl_file_list(
+            folders=test_folders, files_include=files_include, files_avoid=files_avoid
+        )
 
         synthesis_files = self.get_synthesis_files(
             files_include=files_include, files_avoid=files_avoid, **kwargs
         )
-        test_files = self._get_hdl_file_list(
-            test_folders, files_include=files_include, files_avoid=files_avoid
-        )
 
         return synthesis_files + test_files
 
-    def get_formal_files(self, files_include=None, files_avoid=None, **kwargs):
+    def get_documentation_files(
+        self, files_include=None, files_avoid=None, **kwargs
+    ):  # pylint: disable=unused-argument
         """
-        Returns the files to be used for formal verification.
-        By default these are the simulation files, with testbenches excluded
-        (by calling :meth:`get_simulation_files`).
-        Overload this method to select files manually.
+        Get a list of files that shall be included in a documentation build.
 
-        See :meth:`.get_synthesis_files` for instructions on how to use ``files_include``
-        and ``files_avoid``.
-
-        Arguments:
-            files_include (set(`pathlib.Path`)): Optionally filter to only include these files.
-            files_avoid (set(`pathlib.Path`)): Optionally filter to discard these files.
-            kwargs: Further parameters that can be sent by formal flow to control what
-                files are included.
+        It will return all files from the module except testbenches and any generated
+        register package.
+        Overwrite in a child class if you want to change this behavior.
 
         Return:
-            list(HdlFile): Files that should be included in a formal verification project.
+            list(HdlFile): Files that should be included in documentation.
         """
-        return self.get_simulation_files(
-            include_tests=False, files_include=files_include, files_avoid=files_avoid, **kwargs
+        return self._get_hdl_file_list(
+            folders=self.synthesis_folders + self.sim_folders,
+            files_include=files_include,
+            files_avoid=files_avoid,
         )
 
     # pylint: disable=unused-argument
@@ -300,23 +334,7 @@ class BaseModule:
                 location of test files, etc.
         """
 
-    def setup_formal(self, formal_proj, **kwargs):
-        """
-        Setup this module's formal tests.
-
-        .. Note::
-            This default method does nothing. Should be overridden by modules that
-            utilize formal verification.
-
-        Arguments:
-            formal_proj: The formal project that is being used.
-            kwargs: Use this to pass an arbitrary list of arguments from your ``formal.py``
-                to the module where you set up your tests. This could be, e.g., data dimensions,
-                location of test files, etc.
-        """
-
-    # pylint: disable=unused-argument, no-self-use
-    def pre_build(self, project, **kwargs):
+    def pre_build(self, project, **kwargs):  # pylint: disable=unused-argument
         """
         This method hook will be called before an FPGA build is run. A typical use case for this
         mechanism is to set a register constant or default value based on the generics that
@@ -328,7 +346,7 @@ class BaseModule:
             utilize this mechanism.
 
         Arguments:
-            project (.VivadoProject): The project that is being built.
+            project (VivadoProject): The project that is being built.
             kwargs: All other parameters to the build flow. Includes arguments to
                 :meth:`.VivadoProject.build` method as well as other arguments set in
                 :meth:`.VivadoProject.__init__`.
@@ -338,7 +356,7 @@ class BaseModule:
         """
         return True
 
-    def get_build_projects(self):  # pylint: disable=no-self-use
+    def get_build_projects(self):
         """
         Get FPGA build projects defined by this module.
 
@@ -356,7 +374,12 @@ class BaseModule:
         """
         Construct a string suitable for naming test cases.
 
-        Example result: MyName.GenericA_ValueA.GenericB_ValueB
+        Arguments:
+            name (str): Optional base name.
+            generics (dict): Dictionary of values that will be included in the name.
+
+        Returns:
+            str: For example ``MyBaseName.GenericA_ValueA.GenericB_ValueB``.
         """
         if name:
             test_case_name = name
@@ -365,6 +388,7 @@ class BaseModule:
 
         if generics:
             generics_string = ".".join([f"{key}_{value}" for key, value in generics.items()])
+
             if test_case_name:
                 test_case_name = f"{name}.{generics_string}"
             else:
@@ -372,10 +396,50 @@ class BaseModule:
 
         return test_case_name
 
-    def add_vunit_config(self, test, name=None, generics=None, pre_config=None, post_check=None):
+    def add_vunit_config(
+        self,
+        test,
+        name=None,
+        generics=None,
+        set_random_seed=False,
+        pre_config=None,
+        post_check=None,
+    ):  # pylint: disable=too-many-arguments
         """
-        Add config for VUnit test case. Wrapper that sets a suitable name.
+        Add config for VUnit test case. Wrapper that sets a suitable name and can set a random
+        seed generic.
+
+        Arguments:
+            test: VUnit test object. Can be testbench or test case.
+            name (str): Optional designated name for this config. Will be used to form the name of
+                the config together with the ``generics`` value.
+            generics (dict): Generic values that will be applied to the testbench entity. The values
+                will also be used to form the name of the config.
+            set_random_seed (bool, int): Controls setting of the ``seed`` generic:
+
+                * When this argument is not assigned, or assigned ``False``, the generic will not
+                  be set.
+                * When set to boolean ``True``, a random natural (non-negative integer)
+                  generic value will be set.
+                * When set to an integer value, that value will be set for the generic.
+                  This is useful to get a static test case name for waveform inspection.
+
+                If the generic is to be set it must exist in the testbench entity, and should have
+                VHDL type ``natural``.
+            pre_config: Function to be run before the test. See VUnit documentation for details.
+            post_check: Function to be run after the test. See VUnit documentation for details.
         """
+        generics = {} if generics is None else generics
+
+        # Note that "bool" is a sub-class of "int" in python, so isinstance(set_random_seed, int)
+        # returns True if it is an integer or a bool.
+        if isinstance(set_random_seed, bool):
+            if set_random_seed:
+                # Use the maximum range for a natural in VHDL-2008
+                generics["seed"] = random.randint(0, 2**31 - 1)
+        elif isinstance(set_random_seed, int):
+            generics["seed"] = set_random_seed
+
         name = self.test_case_name(name, generics)
         test.add_config(name=name, generics=generics, pre_config=pre_config, post_check=post_check)
 
@@ -399,10 +463,10 @@ def get_modules(
         names_avoid (list(str)): If specified, modules with these names will be discarded.
         library_name_has_lib_suffix (bool): If set, the library name will be
             ``<module name>_lib``, otherwise it is just ``<module name>``.
-        default_registers (list(Register)): Default registers.
+        default_registers (list(hdl_registers.register.Register)): Default registers.
 
     Return:
-        :class:`.ModuleList`: List of module objects (:class:`BaseModule` or child classes thereof)
+        ModuleList: List of module objects (:class:`BaseModule` or child classes thereof)
         created from the specified folders.
     """
     modules = ModuleList()
@@ -439,46 +503,3 @@ def _get_module_object(path, name, library_name_has_lib_suffix, default_register
     if module_file.exists():
         return load_python_module(module_file).Module(path, library_name, default_registers)
     return BaseModule(path, library_name, default_registers)
-
-
-def get_hdl_modules(names_include=None, names_avoid=None):
-    """
-    Wrapper of :func:`.get_modules` which returns the ``hdl_modules`` module objects.
-    If this is a PyPI release of tsfpga, a release of ``hdl_modules`` will be bundled and modules
-    from there will be returned.
-
-    If this is a repo checkout of tsfpga, the function will try to find the ``hdl_modules`` repo
-    if it is next to the tsfpga repo.
-
-    If neither of those alternatives work, the function will assert False.
-
-    Arguments will be passed on to :func:`.get_modules`.
-
-    Return:
-        :class:`.ModuleList`: The module objects.
-    """
-    if tsfpga.HDL_MODULES_LOCATION is not None:
-        if not tsfpga.HDL_MODULES_LOCATION.exists():
-            raise FileNotFoundError(
-                f"The marked location {tsfpga.HDL_MODULES_LOCATION} does not exist. "
-                "There is something wrong with the release script."
-            )
-
-        return get_modules(
-            modules_folders=[tsfpga.HDL_MODULES_LOCATION],
-            names_include=names_include,
-            names_avoid=names_avoid,
-        )
-
-    # Presumed location of the hdl_modules repo
-    hdl_modules_repo_root = (tsfpga.REPO_ROOT.parent / "hdl_modules").resolve()
-    if (hdl_modules_repo_root / "modules").exists():
-        return get_modules(
-            modules_folders=[hdl_modules_repo_root / "modules"],
-            names_include=names_include,
-            names_avoid=names_avoid,
-        )
-
-    raise FileNotFoundError(
-        f"The hdl_modules modules could not be found. Searched in {hdl_modules_repo_root}"
-    )
