@@ -1,11 +1,14 @@
 # --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
 #
-# This file is part of the tsfpga project.
+# This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
 # https://gitlab.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
 
+# Standard libraries
+import copy
+
 
 class ModuleList:
     """
@@ -59,7 +62,7 @@ class ModuleList:
         Create a shallow copy of the module list. This public function is available as a
         convenience and to mimic the interface of a regular python list.
         """
-        return self.__copy__()
+        return copy.copy(self)
 
     def __str__(self):
         return str(self._modules)
