@@ -1,7 +1,7 @@
 # --------------------------------------------------------------------------------------------------
 # Copyright (c) Lukas Vik. All rights reserved.
 #
-# This file is part of the tsfpga project.
+# This file is part of the tsfpga project, a project platform for modern FPGA development.
 # https://tsfpga.com
 # https://gitlab.com/tsfpga/tsfpga
 # --------------------------------------------------------------------------------------------------
@@ -28,4 +28,5 @@ set_property STRATEGY Flow_RuntimeOptimized [get_runs impl_2]
 
 # This option shortens router runtime at the expense of repeatability. With -ultrathreads,
 # the router runs faster but there is a variation in routing between identical runs.
-set_property -name {STEPS.PHYS_OPT_DESIGN.ARGS.MORE OPTIONS} -value -ultrathreads -objects [get_runs impl_2]
+set_property -name {STEPS.PHYS_OPT_DESIGN.ARGS.MORE OPTIONS} -value -ultrathreads \
+  -objects [get_runs impl_2]
